import React, { useState, useEffect, useContext } from "react";
import { withTracker } from "meteor/react-meteor-data";
import { Table, Pagination, Row, Col, Select } from "react-materialize";
import { Mongo } from "meteor/mongo";
import Search from "./Search";
import TableHeader from "./TableHeader";
import { AppContext } from "../contexts/appContext";
import { i18n } from "meteor/universe:i18n";

export const SORT_ASC = 1;
export const SORT_DESC = -1;

//counter is client side collection, data created publish counter
const Counter = new Mongo.Collection("DataTableCounter");
const countRecords = (subsName) => {
  let c = Counter.findOne({ _id: subsName });
  return c ? c.count : 0;
};
//client side mongo collection, data created publish
const DataTableIds = new Mongo.Collection("DataTablePageIds");
const getIds = (id) => {
  const dti = DataTableIds.findOne({ _id: id });
  return dti?.ids || [];
};

// //read settings from storage
// export const loadPageData = (storageKey) => {
//     return sessionStorage.getItem(storageKey)
//         ? JSON.parse(sessionStorage.getItem(storageKey))
//         : {};
// }

const DataTableComponent = (props) => {
  const T = i18n.createComponent();
  const {
    headers,
    renderRow,
    leftFilter,
    //parameter s pre col leftFiler - pre mobil (small), ak nie je je 12 leftFilter aj search su potom pod sebou
    leftFilterColS = 12,
    tableClassName = "",
    searchLabel = i18n.__("messages.search") + " ...",
  } = props;

  const searchColS = leftFilterColS < 12 ? 12 - leftFilterColS : 12;
  const {
    dataLoading,
    rows,
    totalItems,
    sort,
    setSort,
    page,
    setPage,
    pageSize,
    setPageSize,
    fulltext,
    setFulltext,
    updateInitalData,
  } = props.data;

  const pageSizeChanged = (event) => {
    event.preventDefault();
    console.log(event.currentTarget);
    const _pageSize = parseInt(event.currentTarget.value);
    setPageSize(_pageSize);
    setPage(0);
    updateInitalData(0, sort, fulltext, _pageSize);
  };

  const renderSearch = () => {
    return (
      <Search
        search={fulltext}
        onSearch={(value) => {
          if (setPage) {
            setPage(0);
          }
          setFulltext(value);
          updateInitalData(0, sort, value, pageSize);
        }}
        label={searchLabel}
      />
    );
  };

  return (
    <React.Fragment>
      {leftFilter ? (
        <Row>
          <Col m={6} s={leftFilterColS}>
            {leftFilter}
          </Col>
          <Col m={2} s={2}>
            <Select
              s={12}
              id={"pageSize"}
              name={"pageSize"}
              label={i18n.__("messages.show")}
              value={pageSize}
              onChange={pageSizeChanged}
            >
              <option
                key={"10"}
                value={"10"}
                selected={pageSize === 10 ? "selected" : ""}
              >
                10
              </option>
              <option
                key={"25"}
                value={"25"}
                selected={pageSize === 25 ? "selected" : ""}
              >
                25
              </option>
              <option
                key={"50"}
                value={"50"}
                selected={pageSize === 50 ? "selected" : ""}
              >
                50
              </option>
              <option
                key={"100"}
                value={"100"}
                selected={pageSize === 100 ? "selected" : ""}
              >
                100
              </option>
            </Select>
          </Col>
          <Col m={4} s={searchColS}>
            {renderSearch()}
          </Col>
        </Row>
      ) : (
        <Row>
          <Col m={2} s={2}>
            <Select
              s={12}
              id={"pageSize"}
              name={"pageSize"}
              label={i18n.__("messages.show")}
              value={pageSize}
              onChange={pageSizeChanged}
            >
              <option
                key={"3"}
                value={"3"}
                selected={pageSize === 3 ? "selected" : ""}
              >
                3
              </option>
              <option
                key={"10"}
                value={"10"}
                selected={pageSize === 10 ? "selected" : ""}
              >
                10
              </option>
              <option
                key={"25"}
                value={"25"}
                selected={pageSize === 25 ? "selected" : ""}
              >
                25
              </option>
              <option
                key={"50"}
                value={"50"}
                selected={pageSize === 50 ? "selected" : ""}
              >
                50
              </option>
              <option
                key={"100"}
                value={"100"}
                selected={pageSize === 100 ? "selected" : ""}
              >
                100
              </option>
            </Select>
          </Col>
          <Col m={10} s={10}>
            {renderSearch()}
          </Col>
        </Row>
      )}
      {dataLoading ? (
        <div>loading ...</div>
      ) : (
        <div style={{ maxWidth: "100%", overflowX: "auto" }}>
          <Table className={tableClassName || "datatable-table"}>
            <TableHeader
              headers={headers}
              sort={sort}
              onSort={(field) => {
                const newSort = { field: field, order: SORT_ASC };
                if (sort && sort.field === field) {
                  if (sort.order === SORT_DESC) {
                    newSort.field = "";
                  } else {
                    newSort.order = SORT_DESC;
                  }
                }
                if (setPage) {
                  setPage(0);
                }
                setSort(newSort);
                updateInitalData(0, newSort, fulltext, pageSize);
              }}
            />
            <tbody>{rows.map((row) => renderRow(row))}</tbody>
          </Table>
          <Row style={{ width: "calc(100% - 20px)" }}>
            <Col s={12} m={8}>
              {totalItems > pageSize && (
                <Pagination
                  className={"center"}
                  activePage={page + 1}
                  items={Math.ceil(totalItems / pageSize)}
                  leftBtn={<i className="fa fa-chevron-left"></i>}
                  maxButtons={8}
                  rightBtn={<i className="fa fa-chevron-right"></i>}
                  onSelect={(p) => {
                    if (setPage) {
                      setPage(p - 1);
                    }
                    updateInitalData(p - 1, sort, fulltext, pageSize);
                  }}
                />
              )}
            </Col>
            <Col s={12} m={4} style={{ textAlign: "right" }}>
              <T>messages.records_count</T>: {totalItems}
            </Col>
          </Row>
        </div>
      )}
    </React.Fragment>
  );
};

const DataTable = withTracker((props) => {
  const {
    headers,
    renderRow,
    collection,
    subsName,
    defaultInitialSort = null,
    extraFields = null, //array of string,
    refData = null,
    onChangedFulltext = null,
  } = props;
  const leftFilter = props.leftFilter || null;
  const filter = props.filter || {};
  const fulltextFields = props.fulltextFields || [];

  const fields = headers
    .filter((header) => header.field && header.field !== "action")
    .reduce((total, header) => {
      total[header.field] = 1;
      return total;
    }, {});
  if (extraFields) {
    extraFields.forEach((x) => {
      fields[x] = 1;
    });
  }

  //init data from appContext
  const { pageData, setPageData } = useContext(AppContext);
  const initialPage =
    pageData && pageData[subsName] ? pageData[subsName].page : 0;
  const initialPageSize =
    pageData && pageData[subsName] ? pageData[subsName].pageSize : 10;
  const initialFulltext =
    pageData && pageData[subsName] ? pageData[subsName].fulltext : "";
  const initialSort =
    pageData && pageData[subsName]
      ? pageData[subsName].sort
      : defaultInitialSort
      ? defaultInitialSort
      : { field: "", order: SORT_ASC };

  //state constants
  const [sort, setSort] = useState(initialSort);
  const [fulltext, setFulltext] = useState(initialFulltext);
  const [page, setPage] = useState(initialPage);
  const [pageSize, setPageSize] = useState(initialPageSize || 10);
  const sortField = sort && sort.field ? sort.field : "";
  const sortOrder = sort && sort.order ? sort.order : SORT_ASC;
  const query = { _id: { $in: getIds(subsName) } }; ////buildQueryFulltext(selector, fulltext, fulltextFields);
  const sort2 = sortField ? { [sortField]: sortOrder } : {};

  //save data to storage
  const updateInitalData = (pPage, pSort, pFulltext, pPageSize) => {
    const _pageData = {
      page: pPage,
      pageSize: pPageSize,
      sort: pSort,
      fulltext: pFulltext,
    };
    console.log("uppdate page data", _pageData);
    setPageData(subsName, _pageData);
    if (onChangedFulltext) {
      onChangedFulltext(pFulltext);
      setFulltext(pFulltext);
    }
  };

  let isReady = function () {
    //on server must be publish subsName and subsName + 'Count'
    let subs = [
      Meteor.subscribe(
        subsName,
        filter,
        fulltext,
        sortField,
        sortOrder,
        fields,
        page,
        pageSize
      ),
      Meteor.subscribe(subsName + "Count", filter, fulltext),
    ];
    let ready = true;
    subs.forEach((sub) => {
      if (!sub.ready()) ready = false;
    });
    return ready;
  };

  let data = { dataLoading: true };

  if (isReady()) {
    const rows = collection.find(query, { sort: sort2 }).fetch();
    const totalItems = countRecords(subsName);
    data = {
      dataLoading: false,
      rows,
      totalItems,
      sort,
      setSort,
      page,
      setPage,
      pageSize,
      setPageSize,
      fulltext,
      setFulltext,
      updateInitalData,
      onChangedFulltext,
    };
    if (refData) {
      refData.current = data;
    }
  }
  return { data };
})(DataTableComponent);

export default DataTable;
